<?php

/**
 * Compile content of post
 *
 * @package CompileContentOfPost
 * @version 2.1.3
 * @author davidhs
 * @copyright 2013-2014 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License http://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Gets list of settings.
function ccopGetListSettings($start, $items_per_page, $sort)
{
	global $sourcedir;

	if (!is_numeric($start) || empty($items_per_page) || empty($sort))
		return false;

	require_once ($sourcedir . '/Subs-CompileContentOfPost.php');

	$list = array();

	$ccop_settings = ccopGetSettings('all');

	if (!empty($ccop_settings))
	{
		// Applies $sort.
		$sort_array = explode(' ', $sort);
		if (1 == count($sort_array))
			$sort_array[] = 'ASC';

		$flag_order = 'DESC' == strtoupper($sort_array[1]) ? SORT_DESC : SORT_ASC;
		$ccop_settings = ccop_array_column_sort($ccop_settings, $sort_array[0], $flag_order, SORT_REGULAR);

		// Reindex.
		$ccop_settings2 = $ccop_settings;
		$ccop_settings = array();
		foreach ($ccop_settings2 as $row)
			$ccop_settings[] = $row;

		// Paginate.
		$lim = min(count($ccop_settings), $start + $items_per_page);
		for ($i = $start; $i < $lim; $i++)
			$list[] = $ccop_settings[$i];
	}

	return $list;
}

// Gets size of list of settings.
function ccopGetListSettingsSize()
{
	global $sourcedir;

	require_once ($sourcedir . '/Subs-CompileContentOfPost.php');

	$ccop_settings = ccopGetSettings('all');

	return count($ccop_settings);
}

// Admin area settings (edit).
function ccopSettingsEdit()
{
	global $txt, $context, $func, $sourcedir, $modSettings;

	require_once ($sourcedir . '/Subs-CompileContentOfPost.php');

	loadTemplate('CompileContentOfPost');

	// Sort out the context!
	$context['id'] = isset($_GET['id']) ? (int) $_GET['id'] : 0;
	$context['settings_title'] = $context['id'] ? $txt['ccop_edit_title'] : $txt['ccop_add_title'];
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'id';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context['admin_tabs']['tabs']['ccop']['is_selected'] = true;
	$context['sub_template'] = 'ccop_settings_edit';

	if ($context['id'])
		$context['params'] = ccopGetSettings($context['id']);

	// Setup the default values as needed.
	if (empty($context['params']))
		$context['params'] = ccopGetSettings();

	// Others data.
	$context['params']['content'] = $func['htmlspecialchars']($context['params']['content'], ENT_QUOTES);
	$context['params']['boards'] = explode(',', $context['params']['boards']);

	$format_array = ccopSearch();
	$context['params']['type_array'] = array();
	foreach ($format_array as $type => $format)
		$context['params']['type_array'][$type] = $format['name'];

	$context['params']['show_array'] = array(
		'content' => $txt['ccop_show_content'],
		'text' => $txt['ccop_show_text'],
	);

	$context['params']['position_array'] = array(
		'updown' => $txt['ccop_position_updown'],
		'up' => $txt['ccop_position_up'],
		'down' => $txt['ccop_position_down'],
	);

	loadJumpTo();
	$context['params']['all_boards'] = $context['jump_to'];

	// Are we saving?
	if (isset($_POST['save']))
	{
		checkSession('get');

		// Validate inputs.
		$_POST['description'] = $func['htmlspecialchars']($_POST['description'], ENT_QUOTES);

		$boards = empty($_POST['boards']) || in_array('all', $_POST['boards']) ? 'all' : implode(',', $_POST['boards']);

		// Do the insertion/updates.
		$params = array_keys(ccopGetSettings());

		if ($context['id'])
			$id2 = $context['id'];
		else
		{
			$id_new = 1;
			$ids = empty($modSettings['ccop_ids']) ? array() : explode(',', $modSettings['ccop_ids']);
			while (in_array($id_new, $ids))
				$id_new++;
			$id2 = $id_new;
		}

		$setArray = array();
		if (!$context['id'])
		{
			$ids[] = $id_new;
			sort($ids);
			$setArray['ccop_ids'] = implode(',', $ids);
		}
		foreach ($params as $param)
			$setArray['ccop_' . $id2 . '_' . $param] = 'boards' == $param ? $$param : $_POST[$param];
		ccopUpdateSettings($setArray);
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['id'])
	{
		checkSession('get');

		ccopDeleteSettings('ccop\\_' . $context['id'] . '\\_%');

		if (!empty($modSettings['ccop_ids']))
		{
			$ids = explode(',', $modSettings['ccop_ids']);
			$key = array_search($context['id'], $ids);
			if (is_numeric($key))
				unset($ids[$key]);

			$setArray = array();
			$setArray['ccop_ids'] = implode(',', $ids);
			ccopUpdateSettings($setArray);
		}
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=featuresettings;sa=ccop' . $context['params_url']);
	}
}

// Admin area settings (show).
function ccopSettingsShow()
{
	global $txt, $scripturl, $db_prefix, $context, $modSettings, $sourcedir, $settings;

	loadTemplate('CompileContentOfPost');
	loadTemplate('GenericList-CompileContentOfPost');

	$context['sub_template'] = 'ccop_settings_show';
	$context['settings_title'] = $txt['ccop'];

	$base_href = $scripturl . '?action=featuresettings;sa=ccop';

	$get_count = ccopGetListSettingsSize();
	$items_per_page = 25;

	$headers = array(
		'description' => array(
			'label' => $txt['ccop_description'],
			'style' => 'text-align: left;',
			'sort' => array(
				'default' => 'description',
				'reverse' => 'description DESC',
			),
		),
		'type' => array(
			'label' => $txt['ccop_type'],
			'style' => 'width: 30%; text-align: left;',
			'sort' => array(
				'default' => 'type',
				'reverse' => 'type DESC',
			),
		),
		'option_modify' => array(
			'style' => 'width: 10%;',
		),
		'option_delete' => array(
			'style' => 'width: 10%;',
		),
	);

	$sort = isset($_REQUEST['sort']) && isset($headers[$_REQUEST['sort']]) && isset($headers[$_REQUEST['sort']]['sort']) ? $_REQUEST['sort'] : 'description';
	$sort_order = $headers[$sort]['sort'][isset($_REQUEST['desc']) ? 'reverse' : 'default'];
	$page_index = constructPageIndex($base_href . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : ''), $_REQUEST['start'], $get_count, $items_per_page);
	$start = $_REQUEST['start'];
	$params_url = ';start=' . $start . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : '');

	foreach ($headers as $col => $col_header)
	{
		if (isset($headers[$col]['sort']))
		{
			$headers[$col]['href'] = $base_href . ';sort=' . $col;
			if (!isset($_REQUEST['desc']) && $col == $sort)
				$headers[$col]['href'] .= ';desc';
			if ($col == $sort)
				$headers[$col]['sort_image'] = !isset($_REQUEST['desc']) ? 'down' : 'up';
			$headers[$col]['selected'] = $col == $sort;
		}
	}
	$num_columns = count($headers);

	$list = ccopGetListSettings($start, $items_per_page, $sort_order);
	$rows = array();
	foreach ($list as $rowData)
	{
		$textKey = 'ccop_type_' . $rowData['type'];
		$rows[] = array(
			'description' => array(
				'value' => nl2br($rowData['description']),
			),
			'type' => array(
				'value' => isset($txt[$textKey]) ? $txt[$textKey] : $textKey,
			),
			'option_modify' => array(
				'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=ccopedit;id=' . $rowData['id'] . $params_url . '">' . $txt[17] . '</a>',
				'style' => 'text-align: center;',
			),
			'option_delete' => array(
				'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=ccopedit;delete=1;id=' . $rowData['id'] . $params_url . ';sesc=' . $context['session_id'] . '" onclick="return confirm(\'' .$txt['ccop_edit_delete_sure'] . '\');">' . $txt['smf138'] . '</a>',
				'style' => 'text-align: center;',
			),
		);
	}

	$context['ccop_settings_show'] = array(
		'no_items_label' => $txt['ccop_none'],
		'items_per_page' => $items_per_page,
		'page_index' => $page_index,
		'num_columns' => $num_columns,
		'headers' => $headers,
		'rows' => $rows,
		'form' => array(
			'href' => $scripturl . '?action=featuresettings;sa=ccopedit' . $params_url,
			'name' => 'ccop',
			'hidden_fields' => array(
				'sc' => $context['session_id'],
			),
		),
		'additional_rows' => array(
			'below_table_data' => array(
				array(
					'value' => '<input type="submit" name="new" value="' . $txt['ccop_make_new'] . '" />',
					'style' => 'text-align: right;',
				),
			),
		),
	);
}

// Updates settings.
function ccopUpdateSettings($setArray)
{
	global $db_prefix;

	if (empty($setArray))
		return false;

	$replaceArray = array();
	foreach ($setArray as $variable => $value)
		$replaceArray[] = "(SUBSTRING('$variable', 1, 255), SUBSTRING('$value', 1, 65534))";

	db_query("
		REPLACE INTO {$db_prefix}settings
			(variable, value)
		VALUES " . implode(',
			', $replaceArray), __FILE__, __LINE__);

	// Kill the cache - it needs redoing now, but we won't bother ourselves with that here.
	cache_put_data('modSettings', null, 90);
}

// Deletes settings.
function ccopDeleteSettings($filter)
{
	global $db_prefix;

	if (empty($filter))
		return false;

	db_query("
		DELETE FROM {$db_prefix}settings
		WHERE variable LIKE '$filter'", __FILE__, __LINE__);

	// Kill the cache - it needs redoing now, but we won't bother ourselves with that here.
	cache_put_data('modSettings', null, 90);
}

/**
 * array_column_sort
 *
 * function to sort an "arrow of rows" by its columns
 * exracts the columns to be sorted and then
 * uses eval to flexibly apply the standard
 * array_multisort function
 *
 * uses a temporary copy of the array whith "_" prefixed to the keys
 * this makes sure that array_multisort is working with an associative
 * array with string type keys, which in turn ensures that the keys
 * will be preserved.
 *
 * TODO: find a way of modifying the keys of $array directly, without using
 * a copy of the array.
 *
 * flexible syntax:
 * $new_array = array_column_sort($array [, 'col1' [, SORT_FLAG [, SORT_FLAG]]]...);
 *
 * original code credited to Ichier (www.ichier.de) here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 * prefixing array indeces with "_" idea credit to steve at mg-rover dot org, also here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 */
function ccop_array_column_sort() {
	$args = func_get_args();
	$array = array_shift($args);

	// make a temporary copy of array for which will fix the
	// keys to be strings, so that array_multisort() doesn't
	// destroy them
	$array_mod = array();
	foreach ($array as $key => $value) {
		$array_mod['_' . $key] = $value;
	}

	$i = 0;
	$multi_sort_line = "return array_multisort(";
	foreach ($args as $arg) {
		$i++;
		if (is_string($arg)) {
			foreach ($array_mod as $row_key => $row) {
				$sort_array[$i][] = $row[$arg];
			}
		} else {
			$sort_array[$i] = $arg;
		}
		$multi_sort_line .= "\$sort_array[" . $i . "], ";
	}
	$multi_sort_line .= "\$array_mod);";

	eval($multi_sort_line);

	// now copy $array_mod back into $array, stripping off the "_"
	// that we added earlier.
	$array = array();
	foreach ($array_mod as $key => $value) {
		$array[substr($key, 1)] = $value;
	}
	return $array;
}

?>