<?php

/**
 * Compile content of post
 *
 * @package CompileContentOfPost
 * @version 2.1.2
 * @author davidhs
 * @copyright 2013-2014 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License http://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Adds admin area.
function ccop_admin_areas(&$admin_areas)
{
	global $txt;

	$admin_areas['config']['areas']['modsettings']['subsections']['ccop'] = array($txt['ccop']);
}

// Adds modify modification.
function ccop_modify_modifications(&$subActions)
{
	$subActions['ccop'] = 'ccopSettingsShow';
	$subActions['ccopedit'] = 'ccopSettingsEdit';
}

// Gets list of settings.
function ccopGetListSettings($start, $items_per_page, $sort)
{
	if (!is_numeric($start) || empty($items_per_page) || empty($sort))
		return false;

	$list = array();

	$ccop_settings = ccopGetSettings('all');

	if (!empty($ccop_settings))
	{
		// Applies $sort.
		$sort_array = explode(' ', $sort);
		if (1 == count($sort_array))
			$sort_array[] = 'ASC';

		$flag_order = 'DESC' == strtoupper($sort_array[1]) ? SORT_DESC : SORT_ASC;
		$ccop_settings = ccop_array_column_sort($ccop_settings, $sort_array[0], $flag_order, SORT_REGULAR);

		// Reindex.
		$ccop_settings2 = $ccop_settings;
		$ccop_settings = array();
		foreach ($ccop_settings2 as $row)
			$ccop_settings[] = $row;

		// Paginate.
		$lim = min(count($ccop_settings), $start + $items_per_page);
		for ($i = $start; $i < $lim; $i++)
			$list[] = $ccop_settings[$i];
	}

	return $list;
}

// Gets size of list of settings.
function ccopGetListSettingsSize()
{
	$ccop_settings = ccopGetSettings('all');

	return count($ccop_settings);
}

// Admin area settings (edit).
function ccopSettingsEdit($return_config = false)
{
	global $txt, $context, $smcFunc, $sourcedir, $modSettings;

	if ($return_config)
		return array();

	loadTemplate('CompileContentOfPost');

	// Sort out the context!
	$context['id'] = isset($_GET['id']) ? (int) $_GET['id'] : 0;
	$context['settings_title'] = $context['id'] ? $txt['ccop_edit_title'] : $txt['ccop_add_title'];
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'id';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context[$context['admin_menu_name']]['current_subsection'] = 'ccop';
	$context['sub_template'] = 'ccop_settings_edit';

	if ($context['id'])
		$context['params'] = ccopGetSettings($context['id']);

	// Setup the default values as needed.
	if (empty($context['params']))
		$context['params'] = ccopGetSettings();

	// Others data.
	$context['params']['content'] = $smcFunc['htmlspecialchars']($context['params']['content'], ENT_QUOTES);
	$context['params']['boards'] = explode(',', $context['params']['boards']);

	$format_array = ccopSearch();
	$context['params']['type_array'] = array();
	foreach ($format_array as $type => $format)
		$context['params']['type_array'][$type] = $format['name'];

	$context['params']['show_array'] = array(
		'content' => $txt['ccop_show_content'],
		'text' => $txt['ccop_show_text'],
	);

	$context['params']['position_array'] = array(
		'updown' => $txt['ccop_position_updown'],
		'up' => $txt['ccop_position_up'],
		'down' => $txt['ccop_position_down'],
	);

	require_once ($sourcedir . '/Subs-MessageIndex.php');
	$boardListOptions = array(
		'not_redirection' => true,
	);
	$context['params']['all_boards'] = getBoardList($boardListOptions);

	// Are we saving?
	if (isset($_POST['save']))
	{
		checkSession('get');

		// Validate inputs.
		$_POST['description'] = $smcFunc['htmlspecialchars']($_POST['description'], ENT_QUOTES);

		$boards = empty($_POST['boards']) || in_array('all', $_POST['boards']) ? 'all' : implode(',', $_POST['boards']);

		// Do the insertion/updates.
		$params = array_keys(ccopGetSettings());

		if ($context['id'])
			$id2 = $context['id'];
		else
		{
			$id_new = 1;
			$ids = empty($modSettings['ccop_ids']) ? array() : explode(',', $modSettings['ccop_ids']);
			while (in_array($id_new, $ids))
				$id_new++;
			$id2 = $id_new;
		}

		$setArray = array();
		if (!$context['id'])
		{
			$ids[] = $id_new;
			sort($ids);
			$setArray['ccop_ids'] = implode(',', $ids);
		}
		foreach ($params as $param)
			$setArray['ccop_' . $id2 . '_' . $param] = 'boards' == $param ? $$param : $_POST[$param];
		ccopUpdateSettings($setArray);
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['id'])
	{
		checkSession('get');

		ccopDeleteSettings('ccop\\_' . $context['id'] . '\\_%');

		if (!empty($modSettings['ccop_ids']))
		{
			$ids = explode(',', $modSettings['ccop_ids']);
			$key = array_search($context['id'], $ids);
			if (is_numeric($key))
				unset($ids[$key]);

			$setArray = array();
			$setArray['ccop_ids'] = implode(',', $ids);
			ccopUpdateSettings($setArray);
		}
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=admin;area=modsettings;sa=ccop' . $context['params_url']);
	}
}

// Admin area settings (show).
function ccopSettingsShow($return_config = false)
{
	global $txt, $scripturl, $context, $sourcedir;

	if ($return_config)
	{
		$ccop_settings = ccopGetSettings();
		$config_vars = array();
		foreach ($ccop_settings as $param => $v)
			$config_vars[] = array('text', 'ccop_' . $param);

		return $config_vars;
	}

	loadTemplate('CompileContentOfPost');
	require_once ($sourcedir . '/Subs-List.php');

	$context['sub_template'] = 'ccop_settings_show';
	$context['settings_title'] = $txt['ccop'];

	$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
	$sort = isset($_REQUEST['sort']) ? $_REQUEST['sort'] : 'description';
	$params_url = ';start=' . $start . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : '');

	$listOptions = array(
		'id' => 'ccop_settings_show',
		'base_href' => $scripturl . '?action=admin;area=modsettings;sa=ccop',
		'default_sort_col' => 'description',
		'no_items_label' => $txt['ccop_none'],
		'items_per_page' => 25,
		'get_items' => array(
			'function' => 'ccopGetListSettings',
		),
		'get_count' => array(
			'function' => 'ccopGetListSettingsSize',
		),
		'columns' => array(
			'description' => array(
				'header' => array(
					'value' => $txt['ccop_description'],
					'style' => 'text-align: left;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						return nl2br($rowData[\'description\']);
					'),
				),
				'sort' => array(
					'default' => 'description',
					'reverse' => 'description DESC',
				),
			),
			'type' => array(
				'header' => array(
					'value' => $txt['ccop_type'],
					'style' => 'width: 30%; text-align: left;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $txt;

						$textKey = \'ccop_type_\' . $rowData[\'type\'];
						return isset($txt[$textKey]) ? $txt[$textKey] : $textKey;
					'),
				),
				'sort' => array(
					'default' => 'type',
					'reverse' => 'type DESC',
				),
			),
			'option_modify' => array(
				'header' => array(
				),
				'data' => array(
					'sprintf' => array(
						'format' => '<a href="' . $scripturl . '?action=admin;area=modsettings;sa=ccopedit;id=%1$s' . $params_url . '">' . $txt['modify'] . '</a>',
						'params' => array(
							'id' => false,
						),
					),
					'style' => 'width: 10%; text-align: center;',
				),
			),
			'option_delete' => array(
				'header' => array(
				),
				'data' => array(
					'sprintf' => array(
						'format' => '<a href="' . $scripturl . '?action=admin;area=modsettings;sa=ccopedit;delete=1;id=%1$s' . $params_url . ';' . $context['session_var'] . '=' . $context['session_id'] . '" onclick="return confirm(\'' .$txt['ccop_edit_delete_sure'] . '\');">' . $txt['delete'] . '</a>',
						'params' => array(
							'id' => false,
						),
					),
					'style' => 'width: 10%; text-align: center;',
				),
			),
		),
		'form' => array(
			'href' => $scripturl . '?action=admin;area=modsettings;sa=ccopedit' . $params_url,
			'name' => 'ccop',
		),
		'additional_rows' => array(
			array(
				'position' => 'below_table_data',
				'value' => '<input type="submit" name="new" value="' . $txt['ccop_make_new'] . '" class="button_submit" />',
				'style' => 'text-align: right;',
			),
		),
	);
	createList($listOptions);
}

// Updates settings.
function ccopUpdateSettings($setArray)
{
	global $smcFunc;

	if (empty($setArray))
		return false;

	$replaceArray = array();
	foreach ($setArray as $variable => $value)
		$replaceArray[] = array($variable, $value);

	$smcFunc['db_insert']('replace',
		'{db_prefix}settings',
		array('variable' => 'string-255', 'value' => 'string-65534'),
		$replaceArray,
		array('variable')
	);

	// Kill the cache - it needs redoing now, but we won't bother ourselves with that here.
	cache_put_data('modSettings', null, 90);
}

// Deletes settings.
function ccopDeleteSettings($filter)
{
	global $smcFunc;

	if (empty($filter))
		return false;

	$smcFunc['db_query']('', '
		DELETE FROM {db_prefix}settings
		WHERE variable LIKE {string:filter}',
		array(
			'filter' => $filter,
		)
	);

	// Kill the cache - it needs redoing now, but we won't bother ourselves with that here.
	cache_put_data('modSettings', null, 90);
}

/**
 * array_column_sort
 *
 * function to sort an "arrow of rows" by its columns
 * exracts the columns to be sorted and then
 * uses eval to flexibly apply the standard
 * array_multisort function
 *
 * uses a temporary copy of the array whith "_" prefixed to the keys
 * this makes sure that array_multisort is working with an associative
 * array with string type keys, which in turn ensures that the keys
 * will be preserved.
 *
 * TODO: find a way of modifying the keys of $array directly, without using
 * a copy of the array.
 *
 * flexible syntax:
 * $new_array = array_column_sort($array [, 'col1' [, SORT_FLAG [, SORT_FLAG]]]...);
 *
 * original code credited to Ichier (www.ichier.de) here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 * prefixing array indeces with "_" idea credit to steve at mg-rover dot org, also here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 */
function ccop_array_column_sort() {
	$args = func_get_args();
	$array = array_shift($args);

	// make a temporary copy of array for which will fix the
	// keys to be strings, so that array_multisort() doesn't
	// destroy them
	$array_mod = array();
	foreach ($array as $key => $value) {
		$array_mod['_' . $key] = $value;
	}

	$i = 0;
	$multi_sort_line = "return array_multisort(";
	foreach ($args as $arg) {
		$i++;
		if (is_string($arg)) {
			foreach ($array_mod as $row_key => $row) {
				$sort_array[$i][] = $row[$arg];
			}
		} else {
			$sort_array[$i] = $arg;
		}
		$multi_sort_line .= "\$sort_array[" . $i . "], ";
	}
	$multi_sort_line .= "\$array_mod);";

	eval($multi_sort_line);

	// now copy $array_mod back into $array, stripping off the "_"
	// that we added earlier.
	$array = array();
	foreach ($array_mod as $key => $value) {
		$array[substr($key, 1)] = $value;
	}
	return $array;
}

?>