<?php

/**
 * Compile content of post
 *
 * @package CompileContentOfPost
 * @version 2.1
 * @author davidhs
 * @copyright 2013 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License http://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Returns boards and children boards.
function ccopAddChildrenBoards($board_array, $child_boards)
{
	global $smcFunc;

	if ((!is_numeric($child_boards) || !empty($child_boards)) && !empty($board_array))
	{
		$request = $smcFunc['db_query']('', '
			SELECT id_board
			FROM {db_prefix}boards
			WHERE id_parent IN ({array_int:id_parent_array})',
			array('id_parent_array' => $board_array)
		);
		$child_array = array();
		while ($row = $smcFunc['db_fetch_assoc']($request))
			$child_array[] = $row['id_board'];
		$smcFunc['db_free_result']($request);

		if (is_numeric($child_boards))
			$child_boards--;
		$child_array = ccopAddChildrenBoards($child_array, $child_boards);
		$board_array = array_merge($board_array, $child_array);
	}
	return $board_array;
}

// Gets settings.
function ccopGetSettings($id = null)
{
	global $modSettings;

	$default_settings = array(
		'description' => '',
		'content' => '',
		'type' => 'all',
		'show' => 'content',
		'position' => 'updown',
		'boards' => 'all',
		'child_boards' => '0',
	);

	$ids = !empty($modSettings['ccop_ids']) ? explode(',', $modSettings['ccop_ids']) : array();

	$ccop_settings = array();
	if (empty($id))
		$ccop_settings = $default_settings;
	elseif (is_numeric($id))
	{
		if (in_array($id, $ids))
		{
			$ccop_settings['id'] = $id;
			foreach ($default_settings as $param => $v)
				$ccop_settings[$param] = isset($modSettings['ccop_' . $id . '_' . $param]) ? $modSettings['ccop_' . $id . '_' . $param] : $v;
		}
	}
	elseif (!empty($ids)) // $id = all.
	{
		foreach ($ids as $id)
			$ccop_settings[] = ccopGetSettings($id);
	}

	return $ccop_settings;
}

// Returns BBC with compile content of post.
function ccopBbc($body)
{
	global $modSettings, $board;

	loadTemplate('CompileContentOfPost');

	$compilation = array('up' => '', 'down' => '');

	if (!empty($modSettings['ccop_ids']))
	{
		$ids = explode(',', $modSettings['ccop_ids']);
		foreach ($ids as $id)
		{
			// Parameters.
			$ccop_settings = ccopGetSettings($id);
			if (empty($ccop_settings))
				continue;

			extract($ccop_settings);

			if ('all' == $boards)
			{
				if (empty($modSettings['recycle_enable']))
					$do = true;
				else
					$do = $board != $modSettings['recycle_board'];
				
			}
			else
			{
				if (!is_numeric($boards))
				{
					$board_array = array_map('trim', explode(',', $boards));
					$board_array = array_unique($board_array);
					foreach ($board_array as $i => $v)
					{
						if (!is_numeric($v) || empty($v))
							unset($board_array[$i]);
					}
				}
				else
					$board_array = array($boards);

				if (empty($board_array))
					$do = false;
				elseif (in_array($board, $board_array))
					$do = true;
				else
				{
					$board_array = ccopAddChildrenBoards($board_array, $child_boards);
					$do = in_array($board, $board_array);
				}
			}

			// Do compilation.
			if ($do)
			{
				$function = 'ccopBbcType' . ucfirst($type);
				$content = $function($body, $show, $content);
				if (!empty($content))
				{
					if (!(false === strpos($position, 'up')))
						$compilation['up'] .= $content . '<br />';
					if (!(false === strpos($position, 'down')))
						$compilation['down'] .= '<br />' . $content;
				}
			}
		}
	}

	return $compilation;
}

// Returns BBC with content.
function ccopBbcContent($content, $data, $type)
{
	global $settings, $boardurl, $scripturl;

	if (false === strpos($content, '{CONTENT}'))
		$content = '{CONTENT}';

	$trans = array(
		'{CONTENT}' => $data,
		'{TYPE}' => ccopSearch($type, 'name'),
		'{BOARDURL}' => $boardurl,
		'{SCRIPTURL}' => $scripturl,
		'{IMAGES_URL}' => $settings['images_url'],
		'{DEFAULT_IMAGES_URL}' => $settings['default_images_url'],
	);
	$content = strtr($content, $trans);

	return $content;
}

// Returns BBC with compile content of all type.
function ccopBbcTypeAll($body, $show, $content)
{
	return ccopBbcTypeLink($body, $show, $content) . ccopBbcTypeElink($body, $show, $content) . ccopBbcTypeImage($body, $show, $content);
}

// Returns BBC with compile content of link type.
function ccopBbcTypeLink($body, $show, $content)
{
	global $context;

	$link_array = array();
	$pattern_array = ccopSearch('link', 'pattern');
	$pattern0 = array_shift($pattern_array);
	foreach ($pattern_array as $pattern)
	{
		if (preg_match_all($pattern, $body, $parts))
		{
			foreach ($parts[1] as $i => $link)
				$link_array[$link] = $parts[0][$i];
		}
	}
	$body = strtr($body, array('&#039;' => '\'', '&nbsp;' => $context['utf8'] ? "\xC2\xA0" : "\xA0", '&quot;' => '>">', '"' => '<"<', '&lt;' => '<lt<'));
	if (preg_match_all($pattern0, $body, $parts))
	{
		foreach ($parts[1] as $i => $link)
		{
			if (!array_key_exists($link, $link_array))
			{
				foreach ($link_array as $text)
				{
					if (!(false === strpos($text, $link)))
						break 2; // Exit foreach $parts[1].
				}
				$link_array[$link] = $parts[0][$i];
			}
		}
	}

	$text = '';
	if (!empty($link_array))
	{
		$text = template_ccop_get_type_link($link_array, $show);
		$text = ccopBbcContent($content, $text, 'link');
	}

	return $text;
}

// Returns BBC with compile content of elink type.
function ccopBbcTypeElink($body, $show, $content)
{
	$elink_array = array();
	$pattern = ccopSearch('elink', 'pattern');
	if (preg_match_all($pattern, $body, $parts))
	{
		foreach ($parts[1] as $i => $elink)
			$elink_array[$elink] = $parts[2][$i];
	}

	$text = '';
	if (!empty($elink_array))
	{
		$text = template_ccop_get_type_elink($elink_array, $show);
		$text = ccopBbcContent($content, $text, 'elink');
	}

	return $text;
}

// Returns BBC with compile content of image type.
function ccopBbcTypeImage($body, $show, $content)
{
	$img_array = array();
	$pattern_array = ccopSearch('image', 'pattern');
	foreach ($pattern_array as $pattern)
	{
		if (preg_match_all($pattern, $body, $parts))
		{
			foreach ($parts[1] as $i => $src)
				$img_array[$src] = $parts[0][$i];
		}
	}

	$text = '';
	if (!empty($img_array))
	{
		$text = template_ccop_get_type_image($img_array, $show);
		$text = ccopBbcContent($content, $text, 'image');
	}

	return $text;
}

// Returns search data of type and key.
function ccopSearch($type = '', $key = '')
{
	global $txt;
	static $search_data = array();

	if (empty($search_data))
	{
		$search_data = array(
			'all' => array(),
			'link' => array(
				'like' => array('%http://%', '%https://%', '%ftp://%', '%ftps://%'),
				'pattern' => array(
					'~((?:http|https)://[\w\-_%@:|]+(?:\.[\w\-_%]+)*(?::\d+)?(?:/[\w\-_\~%\.@!,\?&;=#(){}+:\'\\\\]*)*[/\w\-_\~%@\?;=#}\\\\])~i',
					'~((?:ftp|ftps)://[\w\-_%@:|]+(?:\.[\w\-_%]+)*(?::\d+)?(?:/[\w\-_\~%\.@,\?&;=#(){}+:\'\\\\]*)*[/\w\-_\~%@\?;=#}\\\\])~i',
					'~\[url\](.+?)\[/url\]~i',
					'~\[url=(.+?)\](.+?)\[/url\]~i',
					'~\[iurl\](.+?)\[/iurl\]~i',
					'~\[iurl=(.+?)\](.+?)\[/iurl\]~i',
					'~&lt;a\s+href=(?:&quot;)?((?:http://|ftp://|https://|ftps://).+?)(?:&quot;)?&gt;(.+?)&lt;/a&gt;~i'
				)
			),
			'elink' => array(
				'like' => array('%ed2k://|file|%'),
				'pattern' => '~(ed2k://\|file\|(.+?)\|.+?)(?:&quot;|"|<|\s|$)~i'
			),
			'image' => array(
				'like' => array('%&lt;img %', '%[img]%'),
				'pattern' => array(
					'~\[img.*?\](.+?)\[/img\]~i',
					'~&lt;img\s+src=(?:&quot;)?((?:http://|ftp://|https://|ftps://).+?)(?:&quot;)?(?:\s+alt=(?:&quot;)?(.*?)(?:&quot;)?)?(?:\s?/)?&gt;~i'
				)
			),
		);

		foreach ($search_data as $type2 => $data)
			$search_data[$type2]['name'] = $txt['ccop_type_' . $type2];
	}

	$return = array();
	if (!empty($type))
	{
		if (array_key_exists($type, $search_data))
		{
			if (!empty($key))
			{
				if (array_key_exists($key, $search_data[$type]))
					$return = $search_data[$type][$key];
			}
			else
				$return = $search_data[$type];
		}
	}
	else
		$return = $search_data;
	return $return;
}

?>